﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Atlas.Models.Service;
using Atlas.Consts;
using Atlas.Logger;
using System.Reflection;
using System.Web.Script.Serialization;
using Atlas.Extensions;
using System.Net;
using Atlas.Helpers;
using Atlas.Exceptions;
using Atlas.Helpers.Service;

namespace Atlas.Controllers.Service
{
    [HandleError]
    public class ServiceController : Controller
    {
        // -- /version
        public ActionResult Version()
        {
            try
            {
                this.LogOperationStarted(MethodInfo.GetCurrentMethod().Name);
                VersionInfo version = new VersionInfo();
                version.version = GlobalConsts.VERSION_INFO;
                JsonResult json = Json(version, JsonRequestBehavior.AllowGet);
                this.LogOperationFinished(MethodInfo.GetCurrentMethod().Name, new JavaScriptSerializer().Serialize(json.Data));
                return json;
            }
            catch (Exception ex)
            {
                MyLogger.Log(ex);
                return new HttpStatusCodeResult(GlobalConsts.ERROR_STATUS_CODE, ex.Message);
            }   //catch            
        }

        // -- /groupID/userName/devices?password=password123
        public ActionResult DeviceDiscovery(string groupID, string userName)
        {
            try
            {
                this.LogOperationStarted(MethodInfo.GetCurrentMethod().Name, groupID, userName);
                var queryString = Request.QueryString;
                string password = queryString.Get("password");
                if (!AuthHelper.AuthenticateUser(userName, password, groupID))
                {
                    throw new WrongAuthException(userName, groupID);
                }   //if
                List<Device> devices = DeviceHelper.GetDevices(userName, AuthHelper.GetClientId(groupID));
                if (devices == null)
                {
                    devices = new List<Device>();
                }   //if
                JsonResult json = Json(new { deviceList = devices }, JsonRequestBehavior.AllowGet);
                this.LogOperationFinished(MethodInfo.GetCurrentMethod().Name, new JavaScriptSerializer().Serialize(json.Data));
                return json;
            }
            catch (WrongAuthException ex)
            {
                MyLogger.Log(ex);
                return new HttpStatusCodeResult(GlobalConsts.BAD_CREDENTIALS_STATUS_CODE);
            }   //catch            
            catch (Exception ex)
            {
                MyLogger.Log(ex);
                return new HttpStatusCodeResult(GlobalConsts.ERROR_STATUS_CODE, ex.Message);
            }   //catch            
        }

        // -- /groupID/userName/positions?password=password123
        public ActionResult PositionRequest(string groupID, string userName)
        {
            try
            {
                this.LogOperationStarted(MethodInfo.GetCurrentMethod().Name, groupID, userName);
                var queryString = Request.QueryString;
                string password = queryString.Get("password");
                if (!AuthHelper.AuthenticateUser(userName, password, groupID))
                {
                    throw new WrongAuthException(userName, groupID);
                }   //if                
                List<Position> positions = PositionHelper.GetPositions(userName, AuthHelper.GetClientId(groupID));
                if (positions == null)
                {
                    positions = new List<Position>();
                }   //if
                JsonResult json = Json(new { positionList = positions }, JsonRequestBehavior.AllowGet);
                this.LogOperationFinished(MethodInfo.GetCurrentMethod().Name, new JavaScriptSerializer().Serialize(json.Data));
                return json;
            }
            catch (WrongAuthException ex)
            {
                MyLogger.Log(ex);
                return new HttpStatusCodeResult(GlobalConsts.BAD_CREDENTIALS_STATUS_CODE);
            }   //catch            
            catch (Exception ex)
            {
                MyLogger.Log(ex);
                return new HttpStatusCodeResult(GlobalConsts.ERROR_STATUS_CODE, ex.Message);
            }   //catch            
        }

        // -- /groupID/userName/positionsWithDistance?password=password123
        public ActionResult PositionWithDistanceRequest(string groupID, string userName)
        {
            try
            {
                this.LogOperationStarted(MethodInfo.GetCurrentMethod().Name, groupID, userName);
                var queryString = Request.QueryString;
                string password = queryString.Get("password");
                if (!AuthHelper.AuthenticateUser(userName, password, groupID))
                {
                    throw new WrongAuthException(userName, groupID);
                }   //if                
                List<PositionWithDistance> positions = PositionHelper.GetPositionsWithDistance(userName, AuthHelper.GetClientId(groupID));
                if (positions == null)
                {
                    positions = new List<PositionWithDistance>();
                }   //if
                JsonResult json = Json(new { positionList = positions }, JsonRequestBehavior.AllowGet);
                this.LogOperationFinished(MethodInfo.GetCurrentMethod().Name, new JavaScriptSerializer().Serialize(json.Data));
                return json;
            }
            catch (WrongAuthException ex)
            {
                MyLogger.Log(ex);
                return new HttpStatusCodeResult(GlobalConsts.BAD_CREDENTIALS_STATUS_CODE);
            }   //catch            
            catch (Exception ex)
            {
                MyLogger.Log(ex);
                return new HttpStatusCodeResult(GlobalConsts.ERROR_STATUS_CODE, ex.Message);
            }   //catch            
        }

        // -- /deviceWithEvents/{deviceId}/{year}/{month}/{day}
        public ActionResult DeviceWithEventsRequest(string deviceId, string year, string month, string day)
        {
            int? deviceIdInt = GlobalHelper.ParseInt(deviceId);
            if (!deviceIdInt.HasValue)
            {
                return new HttpStatusCodeResult(GlobalConsts.ERROR_STATUS_CODE, String.Empty);
            }   //if            
            int yearInt = System.DateTime.Now.Year;
            int monthInt = System.DateTime.Now.Month;
            int dayInt = System.DateTime.Now.Day;
            int? tempYear = GlobalHelper.ParseInt(year);
            int? tempMonth = GlobalHelper.ParseInt(month);
            int? tempDay = GlobalHelper.ParseInt(day);
            if ((tempYear.HasValue) && (tempMonth.HasValue) && (tempDay.HasValue))
            {
                yearInt = tempYear.Value;
                monthInt = tempMonth.Value;
                dayInt = tempDay.Value;
            }   //if
            try
            {
                this.LogOperationStarted(MethodInfo.GetCurrentMethod().Name);
                var queryString = Request.QueryString;
                string password = queryString.Get("password");
                if (!AuthHelper.AuthenticateReportUser(password))
                {
                    throw new WrongAuthException();
                }   //if    
                Device device = DeviceHelper.GetDevice(deviceIdInt.Value);
                if (device == null)
                {
                    device = new Device();                        
                }   //if
                List<DeviceEvent> events = EventsHelper.GetEvents(deviceIdInt.Value, yearInt, monthInt, dayInt);
                if (events == null)
                {
                    events = new List<DeviceEvent>();
                }   //if                
                JsonResult json = Json(new {device, events = events }, JsonRequestBehavior.AllowGet);
                this.LogOperationFinished(MethodInfo.GetCurrentMethod().Name, new JavaScriptSerializer().Serialize(json.Data));
                return json;
            }
            catch (WrongAuthException ex)
            {
                MyLogger.Log(ex);
                return new HttpStatusCodeResult(GlobalConsts.BAD_CREDENTIALS_STATUS_CODE);
            }   //catch            
            catch (Exception ex)
            {
                MyLogger.Log(ex);
                return new HttpStatusCodeResult(GlobalConsts.ERROR_STATUS_CODE, ex.Message);
            }   //catch            
        }

        // -- /deviceWithZaplon/{deviceId}/{year}/{month}/{day}
        public ActionResult DeviceWithZaplonRequest(string deviceId, string year, string month, string day)
        {
            int? deviceIdInt = GlobalHelper.ParseInt(deviceId);
            if (!deviceIdInt.HasValue)
            {
                return new HttpStatusCodeResult(GlobalConsts.ERROR_STATUS_CODE, String.Empty);
            }   //if            
            int yearInt = System.DateTime.Now.Year;
            int monthInt = System.DateTime.Now.Month;
            int dayInt = System.DateTime.Now.Day;
            int? tempYear = GlobalHelper.ParseInt(year);
            int? tempMonth = GlobalHelper.ParseInt(month);
            int? tempDay = GlobalHelper.ParseInt(day);
            if ((tempYear.HasValue) && (tempMonth.HasValue) && (tempDay.HasValue))
            {
                yearInt = tempYear.Value;
                monthInt = tempMonth.Value;
                dayInt = tempDay.Value;
            }   //if
            try
            {
                this.LogOperationStarted(MethodInfo.GetCurrentMethod().Name);
                var queryString = Request.QueryString;
                string password = queryString.Get("password");
                if (!AuthHelper.AuthenticateReportUser(password))
                {
                    throw new WrongAuthException();
                }   //if    
                Device device = DeviceHelper.GetDevice(deviceIdInt.Value);
                if (device == null)
                {
                    device = new Device();
                }   //if
                List<SimpleDeviceEvent> events = EventsHelper.GetSimpleEvents(deviceIdInt.Value, yearInt, monthInt, dayInt);
                if (events == null)
                {
                    events = new List<SimpleDeviceEvent>();
                }   //if                
                JsonResult json = Json(new { device, events = events }, JsonRequestBehavior.AllowGet);
                this.LogOperationFinished(MethodInfo.GetCurrentMethod().Name, new JavaScriptSerializer().Serialize(json.Data));
                return json;
            }
            catch (WrongAuthException ex)
            {
                MyLogger.Log(ex);
                return new HttpStatusCodeResult(GlobalConsts.BAD_CREDENTIALS_STATUS_CODE);
            }   //catch            
            catch (Exception ex)
            {
                MyLogger.Log(ex);
                return new HttpStatusCodeResult(GlobalConsts.ERROR_STATUS_CODE, ex.Message);
            }   //catch            
        }

        // -- /deviceDistance/{deviceId}/{timestampFrom}/{timestampTo}
        public ActionResult DeviceDistanceRequest(string deviceId, string timestampFrom, string timestampTo)
        {
            int? deviceIdInt = GlobalHelper.ParseInt(deviceId);
            long? timestampFromLong = GlobalHelper.ParseLong(timestampFrom);
            long? timestampToLong = GlobalHelper.ParseLong(timestampTo);
            if ((!deviceIdInt.HasValue) || (!timestampFromLong.HasValue) || (!timestampToLong.HasValue))
            {
                return new HttpStatusCodeResult(GlobalConsts.ERROR_STATUS_CODE, String.Empty);
            }   //if            
            try
            {
                this.LogOperationStarted(MethodInfo.GetCurrentMethod().Name);
                var queryString = Request.QueryString;
                string password = queryString.Get("password");
                if (!AuthHelper.AuthenticateReportUser(password))
                {
                    throw new WrongAuthException();
                }   //if    

                JsonResult json = Json(new { distance = EventsHelper.GetDistanceForDevice(deviceIdInt.Value, timestampFromLong.Value, timestampToLong.Value) }, JsonRequestBehavior.AllowGet);
                this.LogOperationFinished(MethodInfo.GetCurrentMethod().Name, new JavaScriptSerializer().Serialize(json.Data));
                return json;
            }
            catch (WrongAuthException ex)
            {
                MyLogger.Log(ex);
                return new HttpStatusCodeResult(GlobalConsts.BAD_CREDENTIALS_STATUS_CODE);
            }   //catch            
            catch (Exception ex)
            {
                MyLogger.Log(ex);
                return new HttpStatusCodeResult(GlobalConsts.ERROR_STATUS_CODE, ex.Message);
            }   //catch            
        }

        // -- /getDevices/{client}/{login}
        public ActionResult DeviceDiscoveryForReport(string client, string login)
        {
            try
            {
                this.LogOperationStarted(MethodInfo.GetCurrentMethod().Name, client, login);
                var queryString = Request.QueryString;
                string password = queryString.Get("password");
                if (!AuthHelper.AuthenticateReportUser(password))
                {
                    throw new WrongAuthException(client, login);
                }   //if
                List<Device> devices = DeviceHelper.GetDevices(login, AuthHelper.GetClientId(client));
                if (devices == null)
                {
                    devices = new List<Device>();
                }   //if
                JsonResult json = Json(new { deviceList = devices }, JsonRequestBehavior.AllowGet);
                this.LogOperationFinished(MethodInfo.GetCurrentMethod().Name, new JavaScriptSerializer().Serialize(json.Data));
                return json;
            }
            catch (WrongAuthException ex)
            {
                MyLogger.Log(ex);
                return new HttpStatusCodeResult(GlobalConsts.BAD_CREDENTIALS_STATUS_CODE);
            }   //catch            
            catch (Exception ex)
            {
                MyLogger.Log(ex);
                return new HttpStatusCodeResult(GlobalConsts.ERROR_STATUS_CODE, ex.Message);
            }   //catch            
        }

        private void LogOperationStarted(string methodName)
        {
            MyLogger.Log(String.Format("Atlas method {0} invoked...", methodName));
        }

        private void LogOperationStarted(string methodName, string groupId, string userName)
        {            
            MyLogger.Log(String.Format("Atlas method {0} invoked... params: groupId - {1}, userName - {2}", methodName, groupId, userName));
        }

        private void LogOperationFinished(string methodName, string result)
        {
            MyLogger.Log(String.Format("Atlas method {0} finished... result - {1}", methodName, result));
        }

    }
}
