﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Atlas.Models;
using Atlas.Models.Service;
using Atlas.Models.DAO;
using Atlas.Logger;
using Atlas.Consts;

namespace Atlas.Helpers.Service
{
    public class PositionHelper
    {
        private static Position ConvertSqlDataToPosition(Message message, IQueryable<MessageData> messageDatas)
        {
            if (message == null)
            {
                return null;
            }   //if
            try
            {
                Position position = new Position();
                position.coordinate = new Coordinate(message.Y, message.X);
                position.dateTime = new Atlas.Models.Service.DateTime(message.GpsTime);
                position.deviceId = message.ObjectId.ToString();
                position.heading = message.VectorAngle;
                position.speed = message.VectorSpeed;
                position.ignitionState = GlobalConsts.IGNITION_STATE_UNKNOWN;
                if (messageDatas != null)
                {
                    foreach (MessageData messageData in messageDatas)
                    {
                        DataSource dataSource = messageData.DataSource;
                        if (dataSource != null)
                        {
                            if ((String.Equals(dataSource.Name, "Zaplon")) || (String.Equals(dataSource.Name, "Zapłon")))
                            {
                                if (messageData.Value == 0)
                                {
                                    position.ignitionState = GlobalConsts.IGNITION_STATE_OFF;
                                }   //if
                                else if (messageData.Value == 1)
                                {
                                    position.ignitionState = GlobalConsts.IGNITION_STATE_ON;
                                }   //if
                            }   //if
                        }   //if
                    }   //for
                }   //if
                return position;
            }   //try
            catch (Exception ex)
            {
                MyLogger.Log(ex);
            }   //catch
            return null;
        }

        private static List<Atlas.Models.Object> GetObjectsList(string userName, int clientId)
        {
            try
            {
                List<Atlas.Models.Object> objects = null;
                if (String.IsNullOrEmpty(userName))
                {
                    MyLogger.Log("Cannot get objects list - empty userName", MyLogger.LogLevel.Error);
                    return objects;
                }   //if
                Login login = AuthHelper.GetLoginObject(userName, clientId);
                if (login == null)
                {
                    MyLogger.Log(String.Format("Cannot get objects list - login object null for userName - {0}, clientId - {1}", userName, clientId), MyLogger.LogLevel.Error);
                    return objects;
                }   //if
                ObjectDAO objectDAO = new ObjectDAO();
                if ((login.RoleId == GlobalConsts.ROLE_ID_ADMIN) || (login.RoleId == GlobalConsts.ROLE_ID_POWER_USER))
                {
                    objects = objectDAO.GetObjectsForClient(clientId);
                }   //if
                else if (login.RoleId == GlobalConsts.ROLE_ID_USER)
                {
                    objects = objectDAO.GetObjectsForClient(clientId, login.LoginId);
                }   //else
                return objects;
            }
            catch (Exception ex)
            {
                MyLogger.Log(ex);
            }   //catch
            return null;
        }

        public static List<Position> GetPositions(string userName, int clientId)
        {
            try
            {
                List<Position> positions = new List<Position>();
                MessagesDAO messageDAO = new MessagesDAO();
                List<Atlas.Models.Object> objects = PositionHelper.GetObjectsList(userName, clientId);
                if (objects == null)
                {
                    MyLogger.Log(String.Format("Cannot get positions - objects list null for clientId - {0}, user name - {1}", clientId, userName), MyLogger.LogLevel.Error);
                    return positions;
                }   //if
                foreach (Atlas.Models.Object sqlObject in objects)
                {
                    Message message = messageDAO.GetLastMessageForClientAndObject(clientId, sqlObject.ObjectId);
                    if (message == null)
                    {
                        MyLogger.Log(String.Format("Cannot get positions - message object null for clientId - {0}, objectId - {1}", clientId, sqlObject.ObjectId), MyLogger.LogLevel.Error);
                        return positions;
                    }   //if
                    IQueryable<MessageData> messageDatas = messageDAO.FindDataByMessageId(message.MessageId);
                    if (messageDatas == null)
                    {
                        MyLogger.Log(String.Format("Getting positions warning - message data null for clientId - {0}, messageId - {1}", clientId, message.MessageId), MyLogger.LogLevel.Warning);
                    }   //if
                    Position position = PositionHelper.ConvertSqlDataToPosition(message, messageDatas);
                    if (position != null)
                    {
                        positions.Add(position);
                    }   //if
                }   //for
                return positions;
            }   //try
            catch (Exception ex)
            {
                MyLogger.Log(ex);
                return null;
            }   //catch
        }

        public static List<PositionWithDistance> GetPositionsWithDistance(string userName, int clientId)
        {
            try
            {
                List<PositionWithDistance> positions = new List<PositionWithDistance>();
                MessagesDAO messageDAO = new MessagesDAO();
                List<Atlas.Models.Object> objects = PositionHelper.GetObjectsList(userName, clientId);
                if (objects == null)
                {
                    MyLogger.Log(String.Format("Cannot get positions - objects list null for clientId - {0}, user name - {1}", clientId, userName), MyLogger.LogLevel.Error);
                    return positions;
                }   //if
                foreach (Atlas.Models.Object sqlObject in objects)
                {
                    Message message = messageDAO.GetLastMessageForClientAndObject(clientId, sqlObject.ObjectId);
                    if (message == null)
                    {
                        MyLogger.Log(String.Format("Cannot get positions - message object null for clientId - {0}, objectId - {1}", clientId, sqlObject.ObjectId), MyLogger.LogLevel.Error);
                        return positions;
                    }   //if
                    IQueryable<MessageData> messageDatas = messageDAO.FindDataByMessageId(message.MessageId);
                    if (messageDatas == null)
                    {
                        MyLogger.Log(String.Format("Getting positions warning - message data null for clientId - {0}, messageId - {1}", clientId, message.MessageId), MyLogger.LogLevel.Warning);
                    }   //if
                    Position position = PositionHelper.ConvertSqlDataToPosition(message, messageDatas);
                    if (position != null)
                    {
                        PositionWithDistance positionWithDistance = new PositionWithDistance(position);
                        TripStopDAO tripStopDAO = new TripStopDAO();
                        positionWithDistance.distance = tripStopDAO.GetGpsDistance(sqlObject.ObjectId);
                        positions.Add(positionWithDistance);
                    }   //if
                }   //for
                return positions;
            }   //try
            catch (Exception ex)
            {
                MyLogger.Log(ex);
                return null;
            }   //catch
        }

    }
}
